{#
 Copyright (C) 2023, CERN
 This software is distributed under the terms of the MIT
 licence, copied verbatim in the file "LICENSE".
 In applying this license, CERN does not waive the privileges and immunities
 granted to it by virtue of its status as Intergovernmental Organization
 or submit itself to any jurisdiction.
#}

{% extends "base.html" %}
{# Like base, but with a search bar in the top nav -#}

{% block navbar_left %}
        <form action="{{ url_for('search') }}" type="GET">
          <input type="text" name="query" value="{{ search_query if search_query is defined else '' }}" placeholder="Search projects" >
        </form>
{% endblock %}

{% block page_content %}
<div style="padding-top:2em"></div>
{% endblock page_content %}
