/*
 * Copyright (C) 2023, CERN
 * This software is distributed under the terms of the MIT
 * licence, copied verbatim in the file "LICENSE".
 * In applying this license, CERN does not waive the privileges and immunities
 * granted to it by virtue of its status as Intergovernmental Organization
 * or submit itself to any jurisdiction.
 */

import $ from 'jquery';
window.jQuery = $;
window.$ = $;


import '@fortawesome/fontawesome-free/js/fontawesome'
import '@fortawesome/fontawesome-free/js/solid'
import '@fortawesome/fontawesome-free/js/regular'
